@@ -3,11 +3,15 @@
 # use lsparse'reset( dirname ) repeately
 # By Lee McLoughlin <lmjm@doc.ic.ac.uk>
 #
-# $Id: lsparse.pl,v 2.5 1994/01/28 18:00:12 lmjm Exp lmjm $
+# $Id: lsparse.pl,v 2.7 1994/06/10 18:28:24 lmjm Exp lmjm $
 # $Log: lsparse.pl,v $
-# Revision 2.5  1994/01/28  18:00:12  lmjm
-# Allow for grasp1.univ-lyon1.fr's D at the start of line for directories.
-# Added CTAN parsing.
+# Revision 2.7  1994/06/10  18:28:24  lmjm
+# Another netware variant.
+# Another dosish system.
+# VM/CMS from Andrew Mc.
+#
+# Revision 2.6  1994/04/29  20:11:06  lmjm
+# Overcome strange handling of $1 near a pattern match.
 #
 # Revision 2.4  1994/01/26  15:43:00  lmjm
 # Added info-mac parser.
@@ -55,7 +59,7 @@ $ls_line = '';
 # returns false if the fstype is unknown.
 sub lsparse'reset
 {
-	$here = $currdir = @_[0];
+	$here = $currdir = $_[0];
 	$now = time;
 	# Vms tends to give FULL pathnames reguardless of where
 	# you generate the dir listing from.
@@ -93,12 +97,16 @@ sub lsparse'line_unix
 	local( $fh ) = @_;
 	local( $non_crud, $perm_denied );
 	local( $d );
+	local( $dir );
 
 	if( eof( $fh ) ){
 		return( "", 0, 0, 0 );
 	}
 
 	while( <$fh> ){
+		# Store listing
+		print main'STORE $_;
+
 		# Stomp on carriage returns
 		s/\015//g;
 
@@ -114,6 +122,7 @@ sub lsparse'line_unix
 		# cannot access .stuff/.cshrc
 		if( m%^(.*)/bin/ls:.*Permission denied% ||
 		   m%^(.*)ls:.*Permission denied% ||
+		   m%^(.*)ls:.*No such file or directory% ||
 		   m%^(.*)(cannot|can not) access % ){
 			if( ! $non_crud ){
 				$non_crud = $1;
@@ -154,7 +163,7 @@ sub lsparse'line_unix
 		}
 		
 
-		if( /^([\-FlrwxsStTdD]{10}).*\D(\d+)\s*(\w\w\w\s+\d+\s*(\d+:\d+|\d\d\d\d))\s+(.*)\n/ ){
+		if( /^([\-FlrwxsStTdDa]{10}).*\D(\d+)\s*(\w\w\w\s+\d+\s*(\d+:\d+|\d\d\d\d))\s+(.*)\n/ ){
 			local( $kind, $size, $lsdate, $file ) = ($1, $2, $3, $5);
 			
 			if( $file eq '.' || $file eq '..' ){
@@ -196,14 +205,15 @@ sub lsparse'line_unix
 		# Match starts of directories.  Try not to match
 		# directories whose names ending in :
 		elsif( /^([\.\/]*.*):$/ && ! /^[dcbsp].*\s.*\s.*:$/ ){
-			if( $1 eq '.' ){
+			$dir = $1;
+			if( $dir eq '.' ){
 				next;
 			}
-			elsif( $1 !~ /^\// ){
-				$currdir = "$here/$1";
+			elsif( $dir !~ /^\// ){
+				$currdir = "$here/$dir";
 			}
 			else {
-				$currdir = "$1";
+				$currdir = "$dir";
 			}
 			$currdir =~ s,/+,/,g;
 			$match = $currdir;
@@ -290,6 +300,9 @@ sub lsparse'line_dls
 	}
 
 	while( <$fh> ){
+		# Store listing
+		print main'STORE $_;
+
 		# Stomp on carriage returns
 		s/\015//g;
 
@@ -370,6 +383,9 @@ sub lsparse'line_netware
 	}
 
 	while( <$fh> ){
+		# Store listing
+		print main'STORE $_;
+
 		# Stomp on carriage returns
 		s/\015//g;
 # Unix vs NetWare:
@@ -380,10 +396,16 @@ sub lsparse'line_netware
 #- [R----F--] jrd                197928       Sep 25 15:19    kermit.exe
 #d [R----F--] jrd                   512       Oct 06 09:31    source
 #d [RWCEAFMS] jrd                   512       Sep 04 14:38    lwp
+# Another netware variant
+#d [R----F-]  1 carl                   512 Mar 12 15:47 txt
+# And another..
+#- [-RWCE-F-] mlm                   11820 Feb  3 93 12:00  drivers.doc
+# And another..
+#-[R----F-]  1 supervis      256 Nov 15 14:21 readme.txt
 
-		if( /^([d|l|\-]\s\[[RWCEAFMS\-]{8}\])\s\w+\s+(\d+)\s*(\w\w\w\s+\d+\s*(\d+:\d+|\d\d\d\d))\s+(.*)\n/) {
+		if( /^([d|l|\-]\s*\[[RWCEAFMS\-]+\])\s+(\d+\s+)?\S+\s+(\d+)\s*(\w\w\w\s+\d+\s*(\d+:\d+|\d\d\d\d))\s+(.*)\n/) {
 			local( $kind, $size, $lsdate, $file ) =
-						 ( $1, $2, $3, $5);
+						 ( $1, $3, $4, $6);
 			if( $file eq '.' || $file eq '..' ){
 				next;
 			}
@@ -426,7 +448,6 @@ sub lsparse'line_netware
 		}
 		else {
 			printf( "Unmatched line: %s", $_ );
-			return( '', 0, 0, 0, 0 );
 		}
 	}
 	return( '', 0, 0, 0, 0 );
@@ -436,14 +457,13 @@ sub lsparse'line_netware
 # into a Unix access number.
 sub netware_to_mode
 {
-	local( $chars ) = @_;
-	local( @kind, $c );
+	local( $kind ) = @_;
+	local( @kind, $c, $k );
 
-	# Split and remove first three characters
+	# Ignore all but the mode characters inside []
+	$k = $kind;
+	$k =~ s,.*\[(.*)\].*,$1,;
 	@kind = split( //, $kind );
-	shift( @kind );		# omit directory "d" field
-	shift( @kind );		# omit space separator
-	shift( @kind );		# omit left square bracket
 	$mode = 0;		# init $mode to no access
 
 	foreach $c ( @kind ){
@@ -453,6 +473,8 @@ sub netware_to_mode
 		}
 	return $mode;
 }
+
+
 # --------------------- parse VMS dir output
 # for each file or directory line found return a tuple of
 # (pathname, size, time, type, mode)
@@ -471,6 +493,9 @@ sub lsparse'line_vms
 	}
 
 	while( <$fh> ){
+		# Store listing
+		print main'STORE $_;
+
 		# Stomp on carriage returns
 		s/\015//g;
 
@@ -617,6 +642,10 @@ sub lsparse'line_dosftp
 			}
 
 			$_ = <$fh>;
+
+			# Store listing
+			print main'STORE $_;
+
 			# Ignore the summary at the end and blank lines
 			if( /^\d+ files?\./ || /^\s+$/ ){
 				next;
@@ -667,6 +696,329 @@ sub lsparse'line_dosftp
 }
 
 
+# --------------------- parse output from a slightly DOS-like dir command
+# for each file or directory line found return a tuple of
+# (pathname, size, time, type, mode)
+# pathname is a full pathname relative to the directory set by reset()
+# size is the size in bytes (this is always 0 for directories)
+# time is a Un*x time value for the file
+# type is "f" for a file, "d" for a directory and
+#         "l linkname" for a symlink
+# 03-16-94  06:29AM       <DIR>          .
+# 03-16-94  06:29AM       <DIR>          ..
+# 04-11-94  11:48PM       <DIR>          creative
+# 03-08-94  07:17AM                 5504 article.xfiles.intro
+# 02-28-94  11:44AM                 3262 article1.gillian.anderson
+
+sub lsparse'line_dosish
+{
+	local( $fh ) = @_;
+
+	while( 1 ){
+		if( eof( $fh ) ){
+			return( "", 0, 0, 0 );
+		}
+
+		$_ = <$fh>;
+
+		# Store listing
+		print main'STORE $_;
+
+		# Ignore blank lines
+		if( /^\s+$/ ){
+			next;
+		}
+
+		# Stomp on carriage returns
+		s/\015//g;
+
+		# I'm about to look at this at lot
+		study;
+
+		if( m,(\d+)-(\d+)-(\d+)\s+(\d+):(\d+)(AM|PM)\s+(\d+|<DIR>)\s+(\S+), ){
+			local( $mon, $day, $yr, $hrs, $min, $ampm, $dir_or_size, $file ) =
+				($1, $2, $3, $4, $5, $6, $7, $8);
+			if( $file eq '.' || $file eq '..' ){
+				next;
+			}
+
+			$hrs += 12 if $ampm =~ /PM/;
+			# TODO: fix hacky 19$yr
+			local( $lsdate ) = "$day-$mon-19$yr $hrs:$min";
+			local( $time ) = &main'lstime_to_time( $lsdate );
+			local( $type ) = ($dir_or_size eq '<DIR>' ? 'd' : 'f');
+			local( $mode ) = 0;
+			local( $size ) = 0;
+
+			$size = $dir_or_size if $dir_or_size =~ /^\d/;
+
+			$currdir =~ s,/+,/,g;
+			$file =~ s,^/$match,,;
+			$file = "/$currdir/$file";
+			$file =~ s,/+,/,g;
+			return( substr( $file, 1 ), $size, $time, $type, $mode );
+		}
+		else {
+			printf( "Unmatched line: %s", $_ );
+		}
+	}
+	return( '', 0, 0, 0, 0 );
+}
+
+# --------------------- parse output from supertcp ftp server
+# for each file or directory line found return a tuple of
+# (pathname, size, time, type, mode)
+# pathname is a full pathname relative to the directory set by reset()
+# size is the size in bytes (this is always 0 for directories)
+# time is a Un*x time value for the file
+# type is "f" for a file, "d" for a directory and
+#         "l linkname" for a symlink
+
+# .               <DIR>           11-16-94        17:16
+# ..              <DIR>           11-16-94        17:16
+# INSTALL         <DIR>           11-16-94        17:17
+# CMT             <DIR>           11-21-94        10:17
+# DESIGN1.DOC          11264      05-11-95        14:20
+# README.TXT            1045      05-10-95        11:01
+# WPKIT1.EXE          960338      06-21-95        17:01
+# CMT.CSV                  0      07-06-95        14:56
+
+# .               <DIR>           11/16/94        17:16
+# ..              <DIR>           11/16/94        17:16
+# INSTALL         <DIR>           11/16/94        17:17
+# CMT             <DIR>           11/21/94        10:17
+# DESIGN1.DOC          11264      05/11/95        14:20
+# README.TXT            1045      05/10/95        11:01
+# WPKIT1.EXE          960338      06/21/95        17:01
+# CMT.CSV                  0      07/06/95        14:56
+
+sub lsparse'line_supertcp
+{
+    local( $fh ) = @_;
+
+    while( 1 ) {
+
+	if( $pending ){
+	    $_ = $pending;
+	    $pending = '';
+	}
+	else {
+	    if( eof( $fh ) ){
+		return( "", 0, 0, 0 );
+	    }
+
+	    $_ = <$fh>;
+
+	    # Store listing
+	    print main'STORE $_;
+
+	    # Ignore the summary at the end and blank lines
+	    if( /^\d+ files?\./ || /^\s+$/ ){
+		next;
+	    }
+	}
+
+	# Stomp on carriage returns
+	s/\015//g;
+	s/\s+$//;
+
+               # I'm about to look at this at lot
+	study;
+
+	local( $file, $dirsize, $date, $time ) = split(" ", $_, 4);
+	local( $mon, $day, $yr ) = split ( /[-\/]/, $date, 3);
+
+	if( defined $file ){
+
+	    next if ( $file eq '..' || $file eq '.' );
+
+	    $pending = $5;
+
+	    local( $lsdate ) = "$day-$mon-$yr $time";
+	    local( $time ) = &main'lstime_to_time( $lsdate );
+            local( $type ) = '?';
+	    local( $mode ) = 0;
+
+            if ( $dirsize eq '<DIR>' ) {
+               $type = 'd';
+               $size = 0;
+            }
+            else {
+               $type = 'f';
+	       $size = $dirsize;
+	       $size =~ s/,//g;
+            }
+
+	    $currdir =~ s,/+,/,g;
+	    $file =~ s,^/$match,,;
+	    $file = "/$currdir/$file";
+	    $file =~ s,/+,/,g;
+
+ 	    return( substr( $file, 1 ), $size, $time, $type, $mode );
+	 }
+	 else {
+	    printf( "Unmatched line: %s", $_ );
+	}
+    }
+
+    return( '', 0, 0, 0, 0 );
+}
+
+# --------------------- parse output from a basic OS2 server
+# for each file or directory line found return a tuple of
+# (pathname, size, time, type, mode)
+# pathname is a full pathname relative to the directory set by reset()
+# size is the size in bytes (this is always 0 for directories)
+# time is a Un*x time value for the file
+# type is "f" for a file, "d" for a directory and
+#         "l linkname" for a symlink
+#                 0           DIR   04-11-95   16:26  .
+#                 0           DIR   04-11-95   16:26  ..
+#                 0           DIR   04-11-95   16:26  ADDRESS
+#               612      A          07-28-95   16:45  air_tra1.bag
+#               195      A          08-09-95   10:23  Alfa1.bag
+#                 0           DIR   04-11-95   16:26  ATTACH
+#               372      A          08-09-95   10:26  Aussie_1.bag
+#            310992                 06-28-94   09:56  INSTALL.EXE
+
+sub lsparse'line_os2
+{
+	local( $fh ) = @_;
+
+	while( 1 ){
+		if( eof( $fh ) ){
+			return( "", 0, 0, 0 );
+		}
+
+		$_ = <$fh>;
+
+		# Store listing
+		print main'STORE $_;
+
+		# Ignore blank lines
+		if( /^\s+$/ ){
+			next;
+		}
+
+		# Stomp on carriage returns
+		s/\015//g;
+
+		# I'm about to look at this at lot
+		study;
+
+		if( m,(\d+)\s+((\S+)\s+)?((\S+)\s+)?(\d+)-(\d+)-(\d+)\s+(\d+):(\d+)\s+(\S+), ){
+			local( $size, $flags, $dir, $mon, $day, $yr, $hrs, $min, $file ) =
+				($1, $3, $5, $6, $7, $8, $9, $10, $11);
+			if( $file eq '.' || $file eq '..' ){
+				next;
+			}
+
+			# Maybe there are no flags just a DIR??
+			if( $flags ne '' && $dir eq '' ){
+				$dir = $flags;
+				$flags = '';
+			}
+
+			# TODO: fix hacky 19$yr
+			local( $lsdate ) = "$day-$mon-19$yr $hrs:$min";
+			local( $time ) = &main'lstime_to_time( $lsdate );
+			local( $type ) = ($dir eq 'DIR' ? 'd' : 'f');
+			local( $mode ) = 0;
+			local( $size ) = 0;
+
+			$size = $dir_or_size if $dir_or_size =~ /^\d/;
+
+			$currdir =~ s,/+,/,g;
+			$file =~ s,^/$match,,;
+			$file = "/$currdir/$file";
+			$file =~ s,/+,/,g;
+			return( substr( $file, 1 ), $size, $time, $type, $mode );
+		}
+		else {
+			printf( "Unmatched line: %s", $_ );
+		}
+	}
+	return( '', 0, 0, 0, 0 );
+}
+
+
+# --------------------- parse output from chameleon ftp server
+# for each file or directory line found return a tuple of
+# (pathname, size, time, type, mode)
+# pathname is a full pathname relative to the directory set by reset()
+# size is the size in bytes (this is always 0 for directories)
+# time is a Un*x time value for the file
+# type is "f" for a file, "d" for a directory and
+#         "l linkname" for a symlink
+sub lsparse'line_chameleon
+{
+    local( $fh ) = @_;
+
+    while( 1 ) {
+
+	if( $pending ){
+	    $_ = $pending;
+	    $pending = '';
+	}
+	else {
+	    if( eof( $fh ) ){
+		return( "", 0, 0, 0 );
+	    }
+
+	    $_ = <$fh>;
+	    # Ignore the summary at the end and blank lines
+	    if( /^\d+ files?\./ || /^\s+$/ ){
+		next;
+	    }
+	}
+
+	# Stomp on carriage returns
+	s/\015//g;
+	s/\s+$//;
+
+               # I'm about to look at this at lot
+	study;
+
+	local( $file, $dirsize, $mon, $day, $yr, $time, $perm )
+	    = split(" ", $_, 7);
+
+	if( defined $file ){
+
+	    next if $file eq '..' || $file eq '.';
+
+	    $pending = $5;
+
+	    local( $lsdate ) = "$day-$mon-$yr $time";
+	    local( $time ) = &main'lstime_to_time( $lsdate );
+            local( $type ) = '?';
+	    local( $mode ) = 0;
+
+            if( $dirsize eq '<DIR>' ){
+               $type = 'd';
+               $size = 0;
+            }
+            else {
+               $type = 'f';
+	       $size = $dirsize;
+	       $size =~ s/,//g;
+            }
+
+	    $currdir =~ s,/+,/,g;
+	    $file =~ s,^/$match,,;
+	    $file = "/$currdir/$file";
+	    $file =~ s,/+,/,g;
+
+ 	    return( substr( $file, 1 ), $size, $time, $type, $mode );
+	 }
+	 else {
+	    printf( "Unmatched line: %s", $_ );
+	}
+    }
+
+    return( '', 0, 0, 0, 0 );
+}
+
+
 # --------------------- parse standard MACOS Unix-like ls output
 # for each file or directory line found return a tuple of
 # (pathname, size, time, type, mode)
@@ -685,6 +1037,9 @@ sub lsparse'line_macos
 	}
 
 	while( <$fh> ){
+		# Store listing
+		print main'STORE $_;
+
 		# Stomp on carriage returns
 		s/\015//g;
 
@@ -742,6 +1097,9 @@ sub lsparse'line_lsparse
 	}
 
 	while( <$fh> ){
+		# Store listing
+		print main'STORE $_;
+
 		if( /^(\S+)\s+(\d+)\s+(\d+)\s+((l\s+)?\S+)\s+(\d+)\n$/ ){
 			# looks good.
 			# note that $type is two words iff it starts with 'l'
@@ -781,6 +1139,9 @@ sub lsparse'line_infomac
 	}
 
 	while( <$fh> ){
+		# Store listing
+		print main'STORE $_;
+
 		next if /^;/;
 		if( /^([l-].)\s*(\d+)\s*(\w\w\w\s+\d+\s*(\d+:\d+|\d\d\d\d))\s+(.*)\n/ ){
 			local( $kind, $size, $lsdate, $file ) = ($1, $2, $3, $5);
@@ -829,6 +1190,9 @@ sub lsparse'line_ctan
 	}
 
 	while( <$fh> ){
+		# Store listing
+		print main'STORE $_;
+
 		if( /^\s*(\d+)\s+(\w\w\w\s+\w\w\w\s+\d+\s+\d+:\d+:\d+\s+\d+)\s+(.*)\n/ ){
 			local( $size, $lsdate, $file ) = ($1, $2, $3);
 			
@@ -844,5 +1208,54 @@ sub lsparse'line_ctan
 	return( '', 0, 0, 0, 0 );
 }
 
+# ------------------------------ VM/CMS
+#
+# DIRACC   EXEC     A1    F    132         84          3  01/25/93  14:49:47
+# DIRUNIX  SCRIPT   A1    V     77       1216         17  01/04/93  20:30:47
+# MAIL     PROFILE  A2    F     80          1          1  10/14/92  16:12:27
+#
+# (pathname, size, time, type, mode)
+# pathname is a full pathname relative to the directory set by reset()
+# size is the size in bytes (this is always 0 for directories)
+# for this we guess that it is record length * nrecords -- usually false
+# time is a Un*x time value for the file -- this is good from the m/f
+# type is always "f" for a file
+
+sub lsparse'line_cms
+{
+	local( $fh ) = @_;
+
+	if( $lsparse'readtime ){
+		alarm( $lsparse'readtime );
+	}
+
+	if( eof( $fh ) ){
+		alarm( 0 );
+		return( "", 0, 0, 0 );
+	}
+	while( <$fh> ){
+		# Store listing
+		print main'STORE $_;
+
+		chop;
+		next unless /\d+\/\d+\/\d+\s+\d+:\d+:\d+/;
+		s/^\s+//;
+
+		# Upper case is so ugly
+		if( ! $lsparse'vms_keep_case ){
+			tr/A-Z/a-z/;
+		}
+
+		local( $fname, $ftype, $fdisk, $rectype, $lrecl, $recs,
+		      $blocks, $ldate, $tod ) = split(/\s+/, $_);
+		return( join('.', ($fname, $ftype, $fdisk)),
+		       $lrecl * $recs, &main'lstime_to_time( "$ldate $tod" ),
+		       'f' );
+	}
+	alarm( 0 );
+	return( '', 0, 0, 0, 0 );
+}
+
+
 # -----
 1;
