@@ -98,10 +98,8 @@ $debug = 0;
 $mv_prog = 'mv -f';
 
 # compress must be able to take the -d arg to cause it to uncompress.
-# Debian Linux has no (real) compress due to the patent issue
-#        $sys_compress_prog = &find_prog( 'compress' ) ||
-#        die "No compress command in path\n";
-$sys_compress_prog = "";
+$sys_compress_prog = &find_prog( 'compress' ) ||
+	die "No compress command in path\n";
 $sys_compress_suffix = 'Z';
 # Like compress gzip must be able to take -d
 if( $gzip_prog = &find_prog( 'gzip' ) ){
@@ -144,9 +142,6 @@ $prod_interval = 60;
 # Put the directory that mirror is actually in at the start of PERLLIB.
 $dir = &real_dir_from_path( $0 );
 unshift( @INC, $dir );
-# Debian Linux stores mirror.defaults in /etc/mirror
-$debian_defs = '/etc/mirror';
-unshift( @INC, $debian_defs ) if -d $debian_defs;
 
 # Make sure that your PERLLIB environment variable can get you
 # all these or that they are installed.
@@ -2616,7 +2611,7 @@ sub transfer_file
 		$f =~ s/($shell_metachars)/\\$1/g;
   		$temp = "$f.$compress_suffix";
   		# Am I doing compress to gzip conversion?
-    		if( $compress_conv_patt && $src_path =~ /$compress_conv_patt/ &&
+   		if( $src_path =~ /$compress_conv_patt/ &&
   		    $compress_suffix eq $gzip_suffix ){
 			$comp = "$sys_compress_prog -d < \"$f\" | $gzip_prog > \"$temp\"";
   		}
@@ -3232,7 +3227,7 @@ sub create_assocs
 
 	&msg( "creating assocs ...\n" ) if $debug > 2;
 	foreach $map ( @assocs ){
-		eval "\$$map = \"\$big_temp/$map.$$\"";
+		eval "\$$map = \"\$big_tmp/$map.$$\"";
 		eval "dbmopen( $map, \$$map, 0644 )";
 	}
 	&msg( "creating assocs done\n" ) if $debug > 2;
@@ -3244,7 +3239,7 @@ sub delete_assocs
 
 	&msg( "deleting assocs ...\n" ) if $debug > 2;
 	foreach $map ( @assocs ){
-		eval "\$$map = \"\$big_temp/$map.$$\"";
+		eval "\$$map = \"\$big_tmp/$map.$$\"";
 		eval "dbmclose( $map )";
 		&unlink_dbm( eval "\$$map" );
 		eval "\%$map = ()";
@@ -3255,9 +3250,8 @@ sub delete_assocs
 sub unlink_dbm
 {
 	local( $file ) = @_;
-	unlink "$file.pag" if -e "$file.pag";
-	unlink "$file.dir" if -e "$file.dir";
-	unlink "$file" if -e "$file";
+	unlink "$file.pag";
+	unlink "$file.dir";
 }
 
 # Chop the tmp file up
